"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Server = void 0;
const lambda_api_1 = __importDefault(require("lambda-api"));
const ApiRequest_1 = require("../model/ApiRequest");
const EndpointInfo_1 = require("../model/reflection/EndpointInfo");
const LogFactory_1 = require("../util/logging/LogFactory");
const timed_1 = require("../util/timed");
const Endpoint_1 = require("./Endpoint");
const MiddlewareRegistry_1 = require("./MiddlewareRegistry");
const OpenApiGenerator_1 = require("./open-api/OpenApiGenerator");
const ControllerLoader_1 = require("./reflection/ControllerLoader");
const DecoratorRegistry_1 = require("./reflection/DecoratorRegistry");
/**
 * Server that discovers routes using decorators on controller
 * classes and methods. Processing of requests is preformed by the
 * `lambda-api` package.
 */
let Server = /** @class */ (() => {
    class Server {
        /**
         * Create a new server.
         *
         * @param appContainer Application container to use to build containers.
         * @param appConfig Application config to pass to `lambda-api`.
         */
        constructor(appContainer, appConfig) {
            this.appContainer = appContainer;
            this.appConfig = appConfig;
            this.logFactory = new LogFactory_1.LogFactory(appConfig);
            this.logger = this.logFactory.getLogger(Server);
            // ensure decorator registry has the right log level (now that we know the app config)
            DecoratorRegistry_1.DecoratorRegistry.setLogger(this.logFactory);
            this.api = lambda_api_1.default(appConfig);
            this._middlewareRegistry = new MiddlewareRegistry_1.MiddlewareRegistry(this.logFactory);
            if (this.appConfig.openApi && this.appConfig.openApi.enabled) {
                this.openApiGenerator = new OpenApiGenerator_1.OpenApiGenerator(this.appConfig, this._middlewareRegistry, this.logFactory);
            }
        }
        get middlewareRegistry() {
            return this._middlewareRegistry;
        }
        /**
         * Configure the `API` instance from the `lambda-api`
         * package.
         *
         * @param handler Function that takes an `API` instance as a parameter.
         */
        configure(handler) {
            handler(this.api);
        }
        /**
         * Scans the specified path for javascript files and loads these into
         * the current runtime. Importing the files will invoke the decorators
         * declared within them. Note: this scans only the top level files.
         *
         * API decorators register controllers, endpoints, configuration and middleware.
         * A series of endpoints are built using the decorator components and registered
         * with the `lambda-api` package routing engine.
         *
         * Controllers and error interceptors registered by decorators are built using
         * an IOC container, which allows dependency injection.
         *
         * OpenAPI endpoints will be registered here, if they are enabled in the app
         * config by setting the `openApi.enabled` flag to true.
         *
         * This method must be called before invoking the `processEvent` method.
         *
         * @param controllersPath (Optional) Paths to the directories that contain controller `js` files.
         *                        Dynamic loading of `injectable` controllers is disabled if undefined
         *                        or the app `Container` instance has its `autoBindInjectable`
         *                        flag set to `false`.
         */
        async discoverAndBuildRoutes(controllersPath) {
            if (this.appContainer.options.autoBindInjectable && controllersPath) {
                for (let path of controllersPath) {
                    this.logger.debug("Loading controllers from path: %s", path);
                    await ControllerLoader_1.ControllerLoader.loadControllers(path, this.logFactory);
                }
            }
            else {
                this.logger.debug("Dynamic loading of injectable controllers disabled");
            }
            if (this.openApiGenerator) {
                this.registerOpenApiEndpoints();
            }
            for (let endpointKey in DecoratorRegistry_1.DecoratorRegistry.Endpoints) {
                if (!DecoratorRegistry_1.DecoratorRegistry.Endpoints.hasOwnProperty(endpointKey)) {
                    continue;
                }
                this.registerEndpoint(DecoratorRegistry_1.DecoratorRegistry.Endpoints[endpointKey]);
            }
        }
        registerOpenApiEndpoints() {
            this.logger.info("Registering OpenAPI endpoints");
            this.registerOpenApiEndpoint("json");
            this.registerOpenApiEndpoint("yml");
        }
        registerOpenApiEndpoint(format) {
            let specEndpoint = new EndpointInfo_1.EndpointInfo(`internal__openapi::${format}`, async () => await this.openApiGenerator.exportOpenApiSpec(format));
            specEndpoint.path = `/open-api.${format}`;
            specEndpoint.httpMethod = "GET";
            specEndpoint.noAuth = !this.appConfig.openApi.useAuthentication;
            specEndpoint.produces = `application/${format}`;
            this.registerEndpoint(specEndpoint);
        }
        registerEndpoint(endpointInfo) {
            let apiEndpoint = new Endpoint_1.Endpoint(endpointInfo, c => this.appContainer.get(c), ei => this.appContainer.get(ei), this._middlewareRegistry, this.logFactory);
            apiEndpoint.register(this.api);
        }
        /**
         * Takes an API request passed in by AWS Lambda and processes
         * it using the `lambda-api` package.
         *
         * @param request API Gateway or ALB request.
         * @param context Request context.
         * @returns The response.
         */
        async processEvent(request, context) {
            let event = request;
            this.logger.info("Processing API request event for path: %s", request.path);
            this.logger.debug("Event data: %j", event);
            this.logger.debug("Event context: %j", context);
            return await this.api.run(event, context);
        }
    }
    __decorate([
        timed_1.timed,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [Array]),
        __metadata("design:returntype", Promise)
    ], Server.prototype, "discoverAndBuildRoutes", null);
    __decorate([
        timed_1.timed,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", [ApiRequest_1.ApiRequest, Object]),
        __metadata("design:returntype", Promise)
    ], Server.prototype, "processEvent", null);
    return Server;
})();
exports.Server = Server;
//# sourceMappingURL=Server.js.map